"""class for the fixed position settings"""

import geocoder

from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QDialogButtonBox, QLineEdit, QPushButton

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
import meshtastic.radioconfig_pb2
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref


class FixedPositionForm(QDialog):
    """fixed position form"""

    def __init__(self, parent=None):
        """constructor"""
        super(FixedPositionForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Fixed Position Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets
        self.fixed_position = QCheckBox()
        self.fixed_position.setToolTip(self.parent.parent.description('fixed_position'))
        self.lat = QLineEdit()
        self.lat.setToolTip(self.parent.parent.description('lat'))
        self.lon = QLineEdit()
        self.lon.setToolTip(self.parent.parent.description('lon'))
        self.alt = QLineEdit()
        self.alt.setToolTip(self.parent.parent.description('alt'))
        self.get_location_using_ip_button = QPushButton("Get Lat/Lon using your IP")
        self.get_location_using_ip_button.clicked.connect(self.latlon)

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.parent.label("fixed_position"), self.fixed_position)
        form_layout.addRow("", self.get_location_using_ip_button)
        form_layout.addRow(self.parent.parent.label("lat"), self.lat)
        form_layout.addRow(self.parent.parent.label("lon"), self.lon)
        form_layout.addRow(self.parent.parent.label("alt"), self.alt)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences

                if self.prefs.fixed_position and self.prefs.fixed_position is True:
                    self.fixed_position.setChecked(True)

        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'fixed_position', f'{self.fixed_position.isChecked()}')
                self.interface.getNode(BROADCAST_ADDR).writeConfig()
                lat = float(self.lat.text())
                lon = float(self.lon.text())
                alt = float(self.alt.text())
                print(f'lat:{lat} lon:{lon} alt:{alt}')
                self.interface.sendPosition(lat, lon, alt)

        except Exception as e:
            print(f'Exception:{e}')


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()


    def latlon(self):
        """Set lat and lon from ip using geocoder"""
        print('set lat and lon from ip using geocoder')
        g = geocoder.ip('me')
        self.lat.setText(f'{g.latlng[0]}')
        self.lon.setText(f'{g.latlng[1]}')



"""class for the serial plugin settings"""


from PySide6 import QtCore
from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QDialogButtonBox, QLineEdit, QLabel

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
import meshtastic.radioconfig_pb2
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref
from meshtastic_flasher.util import zero_if_blank


class SerialForm(QDialog):
    """serial plugin settings form"""

    def __init__(self, parent=None):
        """constructor"""
        super(SerialForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Serial Plugin Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets
        self.serialplugin_about = QLabel(self.parent.parent.doc_url('serialplugin_about'))
        self.serialplugin_about.setOpenExternalLinks(True)
        self.serialplugin_about.setTextFormat(QtCore.Qt.RichText)
        self.serialplugin_about.setToolTip("Link shows more info about the settings for this plugin.")
        self.serialplugin_enabled = QCheckBox()
        self.serialplugin_enabled.setToolTip(self.parent.parent.description('serialplugin_enabled'))
        self.serialplugin_echo = QCheckBox()
        self.serialplugin_echo.setToolTip(self.parent.parent.description('serialplugin_echo'))
        self.serialplugin_mode = QLineEdit()
        self.serialplugin_mode.setToolTip(self.parent.parent.description('serialplugin_mode'))
        self.serialplugin_rxd = QLineEdit()
        self.serialplugin_rxd.setToolTip(self.parent.parent.description('serialplugin_rxd'))
        self.serialplugin_txd = QLineEdit()
        self.serialplugin_txd.setToolTip(self.parent.parent.description('serialplugin_txd'))
        self.serialplugin_timeout = QLineEdit()
        self.serialplugin_timeout.setToolTip(self.parent.parent.description('serialplugin_timeout'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.parent.label("serialplugin_about"), self.serialplugin_about)
        form_layout.addRow(self.parent.parent.label("serialplugin_enabled"), self.serialplugin_enabled)
        form_layout.addRow(self.parent.parent.label("serialplugin_echo"), self.serialplugin_echo)
        form_layout.addRow(self.parent.parent.label("serialplugin_mode"), self.serialplugin_mode)
        form_layout.addRow(self.parent.parent.label("serialplugin_txd"), self.serialplugin_txd)
        form_layout.addRow(self.parent.parent.label("serialplugin_rxd"), self.serialplugin_rxd)
        form_layout.addRow(self.parent.parent.label("serialplugin_timeout"), self.serialplugin_timeout)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences

                if self.prefs.serialplugin_enabled and self.prefs.serialplugin_enabled is True:
                    self.serialplugin_enabled.setChecked(True)

                if self.prefs.serialplugin_echo and self.prefs.serialplugin_echo is True:
                    self.serialplugin_enabled.setChecked(True)

                if self.prefs.serialplugin_mode:
                    self.serialplugin_mode.setText(f'{self.prefs.serialplugin_mode}')
                else:
                    self.serialplugin_mode.setText("0")

                if self.prefs.serialplugin_rxd:
                    self.serialplugin_rxd.setText(f'{self.prefs.serialplugin_rxd}')
                else:
                    self.serialplugin_rxd.setText("0")

                if self.prefs.serialplugin_timeout:
                    self.serialplugin_timeout.setText(f'{self.prefs.serialplugin_timeout}')
                else:
                    self.serialplugin_timeout.setText("0")

                if self.prefs.serialplugin_txd:
                    self.serialplugin_txd.setText(f'{self.prefs.serialplugin_txd}')
                else:
                    self.serialplugin_txd.setText("0")

        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'serialplugin_enabled', f'{self.serialplugin_enabled.isChecked()}')
                setPref(prefs, 'serialplugin_echo', f'{self.serialplugin_echo.isChecked()}')
                setPref(prefs, 'serialplugin_mode', zero_if_blank(self.serialplugin_mode.text()))
                setPref(prefs, 'serialplugin_rxd', zero_if_blank(self.serialplugin_rxd.text()))
                setPref(prefs, 'serialplugin_txd', zero_if_blank(self.serialplugin_txd.text()))
                setPref(prefs, 'serialplugin_timeout', zero_if_blank(self.serialplugin_timeout.text()))
                self.interface.getNode(BROADCAST_ADDR).writeConfig()

        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()



"""class for the Advanced options form"""

import datetime
import glob
import os
import shutil
import sys

from PySide6.QtWidgets import QPushButton, QDialog, QCheckBox, QFormLayout, QFileDialog, QMessageBox

from meshtastic.__main__ import main

from meshtastic_flasher.info_form import InfoForm
from meshtastic_flasher.send_text_form import SendTextForm


class AdvancedForm(QDialog):
    """Advanced options form"""

    def __init__(self, parent=None):
        """constructor"""
        super(AdvancedForm, self).__init__(parent)

        self.info_form = InfoForm()
        self.send_text_form = SendTextForm(self)
        self.parent = parent

        width = 240
        height = 120
        self.setMinimumSize(width, height)
        self.setWindowTitle("Advanced Options")

        # Create widgets
        self.update_only_cb = QCheckBox()
        self.update_only_cb.setToolTip("If enabled, the device will be updated (not completely erased).")
        self.info_button = QPushButton("INFO")
        self.info_button.clicked.connect(self.info)
        self.send_text_button = QPushButton("SEND TEXT")
        self.send_text_button.clicked.connect(self.send_text)
        self.configure_from_file_button = QPushButton("Configure From File")
        self.configure_from_file_button.clicked.connect(self.configure_from_file)
        self.export_configuration_button = QPushButton("Export Configuration")
        self.export_configuration_button.clicked.connect(self.export_configuration)
        self.clear_firmware_files_button = QPushButton("Clear Firmware Files")
        self.clear_firmware_files_button.clicked.connect(self.clear_firmware_files)

        self.ok_button = QPushButton("OK")

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.tr("&Update only"), self.update_only_cb)
        form_layout.addRow(self.tr(""), self.info_button)
        form_layout.addRow(self.tr(""), self.send_text_button)
        form_layout.addRow(self.tr(""), self.export_configuration_button)
        form_layout.addRow(self.tr(""), self.configure_from_file_button)
        form_layout.addRow(self.tr(""), self.clear_firmware_files_button)
        form_layout.addRow(self.tr(""), self.ok_button)
        self.setLayout(form_layout)

        self.ok_button.clicked.connect(self.close_advanced_options)


    def close_advanced_options(self):
        """Close the advanced options form"""
        print('OK button was clicked in advanced options')
        self.close()


    def configure_from_file(self):
        """Configure from file"""
        print('configure from file clicked')
        filename, _ = QFileDialog.getOpenFileName(self, "Open file", "", "YAML Files (*.yaml *.yml);;" "All Files (*.*)")
        if filename:
            old_sys_argv = sys.argv
            sys.argv = ['', '--configure', filename]
            main()
            sys.argv = old_sys_argv
            QMessageBox.information(self, "Info", f"Device configured from:{filename}")


    def export_configuration(self):
        """Export configuration"""
        print('export configuration clicked')
        now = datetime.datetime.now()
        now_formatted = now.strftime("%Y%m%d_%H%M%S")
        filename = f'{now_formatted}_backup.yaml'
        print(f'filename:{filename}')
        old_sys_argv = sys.argv
        old_sys_stdout = sys.stdout
        # pylint: disable=consider-using-with
        sys.stdout = open(filename, 'w', encoding='utf-8')
        sys.argv = ['', '--export-config']
        try:
            main()
        except SystemExit:
            pass
        sys.stdout.close()
        sys.argv = old_sys_argv
        sys.stdout = old_sys_stdout
        QMessageBox.information(self, "Info", f"Device configuration backed up to:{filename}")


    def send_text(self):
        """send_text clicked"""
        self.send_text_form.run()


    def info(self):
        """meshtastic --info"""
        print('info clicked')
        self.info_form.text.clear()
        old_sys_argv = sys.argv
        old_sys_stdout = sys.stdout
        sys.stdout = self.info_form
        sys.argv = ['', '--info']
        try:
            main()
        except SystemExit:
            pass
        sys.argv = old_sys_argv
        sys.stdout = old_sys_stdout
        self.info_form.show()


    def clear_firmware_files(self):
        """clear firmware files"""
        print('clear firmware files')
        confirm_msg = 'Are you sure you want to remove the firmware files?'
        reply = QMessageBox.question(self, 'Confirm', confirm_msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if reply == QMessageBox.Yes:
            print("User confirmed they want remove the firmware files")
            directories = glob.glob('1.2.*.*')
            for directory in directories:
                print(f'removing directory:{directory}')
                shutil.rmtree(directory)
            files = glob.glob('firmware-*.zip')
            for file in files:
                print(f'removing file:{file}')
                os.remove(file)
            # clean up the firmware dropdown
            if self.parent:
                self.parent.firmware_version = None
                self.parent.select_firmware_version.clear()
                self.parent.select_firmware_version.setDisabled(True)
        else:
            print("User does not want remove the firmware files")



"""class for the settings"""


from PySide6.QtWidgets import QTabWidget, QMainWindow, QMessageBox

import meshtastic.serial_interface

from meshtastic_flasher.admin_form import AdminForm
from meshtastic_flasher.wifi_and_mqtt_form import Wifi_and_MQTT_Form
from meshtastic_flasher.user_form import UserForm
from meshtastic_flasher.position_form import PositionForm
from meshtastic_flasher.power_form import PowerForm
from meshtastic_flasher.radio_form import RadioForm
from meshtastic_flasher.channel_settings import ChannelSettings
from meshtastic_flasher.plugin_settings import PluginSettings

from meshtastic_flasher.util import load_fields


class Settings(QMainWindow):
    """settings"""

    def __init__(self):
        """constructor"""
        super(Settings, self).__init__()

        self.port = None
        self.interface = None
        self.pixel_mult = 12

        self.fields = load_fields()

        width = 800
        height = 700
        self.setMinimumSize(width, height)
        self.setWindowTitle("Settings")

        self.admin_form = AdminForm(self)
        self.wifi_and_mqtt_form = Wifi_and_MQTT_Form(self)
        self.user_form = UserForm(self)
        self.position_form = PositionForm(self)
        self.power_form = PowerForm(self)
        self.radio_form = RadioForm(self)
        self.plugin_settings = PluginSettings(self)
        self.channel_settings = ChannelSettings(self)

        self.tabs = QTabWidget()

        self.setStyleSheet("""
QTabWidget::pane {
    position: absolute;
    top: 1em;
}
QTabWidget::tab-bar {
    alignment: center;
}
QTabWidget::tab {
}
QTabBar::tab:selected {
    margin-left: 2px;
    margin-right: 2px;
}
QTabBar::tab:first:selected {
    margin-left: 0; /* the first selected tab has nothing to overlap with on the left */
}
QTabBar::tab:last:selected {
    margin-right: 0; /* the last selected tab has nothing to overlap with on the right */
}
QTabBar::tab:only-one {
    margin: 0; /* if there is only one tab, we don't want overlapping margins */
}
""")
        self.tabs.blockSignals(True) # just for not showing initial message
        self.tabs.currentChanged.connect(self.on_change_tabs)

        self.tabs.setTabPosition(QTabWidget.North)

        self.tabs.addTab(self.user_form, "User")
        self.tabs.addTab(self.wifi_and_mqtt_form, "Wifi/MQTT")
        self.tabs.addTab(self.position_form, "Position")
        self.tabs.addTab(self.power_form, "Power")
        self.tabs.addTab(self.radio_form, "Radio")
        self.tabs.addTab(self.plugin_settings, "Plugins")
        self.tabs.addTab(self.channel_settings, "Channels")
        self.tabs.addTab(self.admin_form, "ADMIN")

        self.setCentralWidget(self.tabs)

        self.tabs.blockSignals(False) # now listen the currentChanged signal



    def label(self, field):
        """Return the label for a field"""
        retval = ""
        if self.fields:
            if field in self.fields:
                if 'label' in self.fields[field]:
                    retval = self.fields[field]['label']
        return retval


    def description(self, field):
        """Return the description for a field"""
        retval = ""
        if self.fields:
            if field in self.fields:
                if 'description' in self.fields[field]:
                    retval = self.fields[field]['description']
        return retval


    def doc_url(self, field):
        """Return the doc_url enriched with html for a field"""
        retval = ""
        if self.fields:
            if field in self.fields:
                if 'doc_url' in self.fields[field]:
                    doc_url = self.fields[field]['doc_url']
                    retval = f"<a href='{doc_url}' style='color:#67EA94'>More info</a>"
        return retval


    def max_size(self, field):
        """Return the max_size for a field"""
        retval = 0
        if self.fields:
            if field in self.fields:
                if 'max_size' in self.fields[field]:
                    retval = self.fields[field]['max_size']
        return retval


    def on_change_tabs(self, i):
        """On change of each tab """
        print(f'on_change_tabs:{i}')
        if i == 0:
            print('user run()')
            self.user_form.run(port=self.port, interface=self.interface)
        elif i == 1:
            print('wifi_and_mqtt_form run()')
            self.wifi_and_mqtt_form.run(port=self.port, interface=self.interface)
        elif i == 2:
            print('position run()')
            self.position_form.run(port=self.port, interface=self.interface)
        elif i == 3:
            print('power run()')
            self.power_form.run(port=self.port, interface=self.interface)
        elif i == 4:
            print('radio run()')
            self.radio_form.run(port=self.port, interface=self.interface)
        elif i == 5:
            print('plugin settings run()')
            self.plugin_settings.run(port=self.port, interface=self.interface)
        elif i == 6:
            print('channel settings run()')
            self.channel_settings.run(port=self.port, interface=self.interface)
        elif i == 7:
            print('admin form run()')
            self.admin_form.run(port=self.port, interface=self.interface)


    def my_close(self):
        """Close this window"""
        if self.port:
            self.port = None
        if self.interface:
            self.interface.close()
            self.interface = None # so any saved values are re-read upon next form use
        self.close()


    # pylint: disable=unused-argument
    def closeEvent(self, event):
        """On close of the Settings window"""
        print('closed Settings')
        self.my_close()


    def run(self):
        """load the form"""
        print('in settings')
        if self.interface is None:
            try:
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
                self.port = self.interface.devPath
                print(f'self.port:{self.port}:')
            except Exception as e:
                print(f'Exception:{e}')
            except SystemExit:
                pass
        if self.interface:
            self.show()
            self.user_form.run(port=self.port, interface=self.interface)
        else:
            QMessageBox.warning(self, "Warning", ("There was a problem connecting to the device.\n\n"
                                "Perhaps unplug device and/or restart this program."))
            print("closing")
            self.my_close()



"""class for the plugin external notifications settings"""


from PySide6 import QtCore
from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QDialogButtonBox, QLineEdit, QLabel

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
import meshtastic.radioconfig_pb2
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref
from meshtastic_flasher.util import zero_if_blank


class ExternalNotificationsForm(QDialog):
    """plugin external notifications form"""

    def __init__(self, parent=None):
        """constructor"""
        super(ExternalNotificationsForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Plugins External Notifications Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets
        self.ext_notification_about = QLabel(self.parent.parent.doc_url('ext_notification_plugin_about'))
        self.ext_notification_about.setOpenExternalLinks(True)
        self.ext_notification_about.setTextFormat(QtCore.Qt.RichText)
        self.ext_notification_about.setToolTip("Link shows more info about the settings for this plugin.")
        self.ext_notification_plugin_enabled = QCheckBox()
        self.ext_notification_plugin_enabled.setToolTip(self.parent.parent.description('ext_notification_plugin_enabled'))
        self.ext_notification_plugin_active = QCheckBox()
        self.ext_notification_plugin_active.setToolTip(self.parent.parent.description('ext_notification_plugin_active'))
        self.ext_notification_plugin_alert_bell = QCheckBox()
        self.ext_notification_plugin_alert_bell.setToolTip(self.parent.parent.description('ext_notification_plugin_alert_bell'))
        self.ext_notification_plugin_alert_message = QCheckBox()
        self.ext_notification_plugin_alert_message.setToolTip(self.parent.parent.description('ext_notification_plugin_alert_message'))
        self.ext_notification_plugin_output = QLineEdit()
        self.ext_notification_plugin_output.setToolTip(self.parent.parent.description('ext_notification_plugin_output'))
        self.ext_notification_plugin_output_ms = QLineEdit()
        self.ext_notification_plugin_output_ms.setToolTip(self.parent.parent.description('ext_notification_plugin_output_ms'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_about"), self.ext_notification_about)
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_enabled"), self.ext_notification_plugin_enabled)
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_active"), self.ext_notification_plugin_active)
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_alert_bell"), self.ext_notification_plugin_alert_bell)
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_alert_message"), self.ext_notification_plugin_alert_message)
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_output"), self.ext_notification_plugin_output)
        form_layout.addRow(self.parent.parent.label("ext_notification_plugin_output_ms"), self.ext_notification_plugin_output_ms)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences

                if self.prefs.ext_notification_plugin_active and self.prefs.ext_notification_plugin_active is True:
                    self.ext_notification_plugin_active.setChecked(True)

                if self.prefs.ext_notification_plugin_alert_bell and self.prefs.ext_notification_plugin_alert_bell is True:
                    self.ext_notification_plugin_alert_bell.setChecked(True)

                if self.prefs.ext_notification_plugin_alert_message and self.prefs.ext_notification_plugin_alert_message is True:
                    self.ext_notification_plugin_alert_message.setChecked(True)

                if self.prefs.ext_notification_plugin_enabled and self.prefs.ext_notification_plugin_enabled is True:
                    self.ext_notification_plugin_enabled.setChecked(True)

                if self.prefs.ext_notification_plugin_output:
                    self.ext_notification_plugin_output.setText(f'{self.prefs.ext_notification_plugin_output}')
                else:
                    self.ext_notification_plugin_output.setText("0")

                if self.prefs.ext_notification_plugin_output_ms:
                    self.ext_notification_plugin_output_ms.setText(f'{self.prefs.ext_notification_plugin_output_ms}')
                else:
                    self.ext_notification_plugin_output_ms.setText("0")

        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'ext_notification_plugin_active', f'{self.ext_notification_plugin_active.isChecked()}')
                setPref(prefs, 'ext_notification_plugin_alert_bell', f'{self.ext_notification_plugin_alert_bell.isChecked()}')
                setPref(prefs, 'ext_notification_plugin_alert_message', f'{self.ext_notification_plugin_alert_message.isChecked()}')
                setPref(prefs, 'ext_notification_plugin_enabled', f'{self.ext_notification_plugin_enabled.isChecked()}')
                setPref(prefs, 'ext_notification_plugin_output', zero_if_blank(self.ext_notification_plugin_output.text()))
                setPref(prefs, 'ext_notification_plugin_output_ms', zero_if_blank(self.ext_notification_plugin_output_ms.text()))
                self.interface.getNode(BROADCAST_ADDR).writeConfig()

        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()



"""class for the send text form"""

import sys

from PySide6.QtWidgets import QDialog, QFormLayout, QDialogButtonBox, QLineEdit, QMessageBox

from meshtastic.__main__ import main


class SendTextForm(QDialog):
    """send text form"""

    def __init__(self, parent=None):
        """constructor"""
        super(SendTextForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Send Text")

        self.prefs = None

        # Create widgets
        self.message = QLineEdit()
        self.message.setToolTip('The message to be sent.')

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Ok)
        self.button_box.rejected.connect(self.rejected)
        self.button_box.accepted.connect(self.send_message)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow("Message", self.message)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self):
        """load the form"""
        self.show()


    def send_message(self):
        """Send the message"""
        print(f'Send {self.message.text()}')

        if self.message.text() == '':
            QMessageBox.information(self, "Info", "Please supply a message to send.")
        else:
            old_sys_argv = sys.argv
            sys.argv = ['', '--sendtext', f'{self.message.text()}']
            try:
                main()
            except SystemExit:
                pass
            sys.argv = old_sys_argv
            QMessageBox.information(self, "Info", "Message sent")
            self.close()


    def rejected(self):
        """Close this form"""
        self.close()



"""class for info form"""

from PySide6.QtWidgets import QDialog, QFormLayout, QPlainTextEdit


class InfoForm(QDialog):
    """info form"""

    def __init__(self, parent=None):
        """constructor"""
        super(InfoForm, self).__init__(parent)

        width = 800
        height = 900
        self.setMinimumSize(width, height)
        self.setWindowTitle("Info")

        # Create widgets
        self.text = QPlainTextEdit()
        self.text.setReadOnly(True)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.text)
        self.setLayout(form_layout)

    def write(self, data):
        """Write the output to text widget"""
        if data:
            self.text.appendPlainText(data)



"""class for the channels settings"""


from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QComboBox, QDialogButtonBox, QLineEdit, QMessageBox, QPushButton

from meshtastic.util import genPSK256, fromPSK
from meshtastic.__init__ import BROADCAST_ADDR

from meshtastic.channel_pb2 import ChannelSettings, Channel
from meshtastic_flasher.util import zero_if_blank

class ChannelForm(QDialog):
    """channels settings form"""

    def __init__(self, parent=None, channel_index=0):
        """constructor"""
        super(ChannelForm, self).__init__(parent)

        self.parent = parent
        self.channel_index = channel_index
        self.use_name = None

        width = 500
        height = 800
        self.setMinimumSize(width, height)
        self.setWindowTitle("Channel Settings")

        self.port = None
        self.interface = None
        self.ch = None
        self.psk = None

        # Create widgets
        self.name = QLineEdit()
        self.name.setToolTip(self.parent.parent.description('name'))
        self.role = QComboBox()
        self.role.setToolTip(self.parent.parent.description('role'))
        self.role.setMinimumContentsLength(17)
        self.modem_config = None
        if self.channel_index == 0:
            self.name.setReadOnly(True) # primary channel name cannot be changed
            # modem config is only on primary channel
            self.modem_config = QComboBox()
            self.modem_config.setToolTip(self.parent.parent.description('modem_config'))
            self.modem_config.setMinimumContentsLength(17)
        self.uplink_enabled = QCheckBox()
        self.uplink_enabled.setToolTip(self.parent.parent.description('uplink_enabled'))
        self.downlink_enabled = QCheckBox()
        self.downlink_enabled.setToolTip(self.parent.parent.description('downlink_enabled'))
        self.tx_power = QLineEdit()
        self.tx_power.setToolTip(self.parent.parent.description('tx_power'))
        self.bandwidth = QLineEdit()
        self.bandwidth.setToolTip(self.parent.parent.description('bandwidth'))
        self.spread_factor = QLineEdit()
        self.spread_factor.setToolTip(self.parent.parent.description('spread_factor'))
        self.coding_rate = QLineEdit()
        self.coding_rate.setToolTip(self.parent.parent.description('coding_rate'))
        self.psk_random_button = QPushButton("PSKRandom")
        self.psk_default_button = QPushButton("PSKDefault")
        if self.channel_index > 0:
            self.delete_this_channel_button = QPushButton("DeleteThisChannel")
            self.delete_this_channel_button.clicked.connect(self.delete_this_channel)
        # TODO: self.id = QLineEdit()

        self.psk_random_button.clicked.connect(self.psk_random)
        self.psk_default_button.clicked.connect(self.psk_default)

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.parent.label("name"), self.name)
        form_layout.addRow(self.parent.parent.label("role"), self.role)
        if self.channel_index == 0:
            form_layout.addRow(self.parent.parent.label("modem_config"), self.modem_config)
        form_layout.addRow(self.parent.parent.label("uplink_enabled"), self.uplink_enabled)
        form_layout.addRow(self.parent.parent.label("downlink_enabled"), self.downlink_enabled)
        form_layout.addRow(self.parent.parent.label("tx_power"), self.tx_power)
        form_layout.addRow(self.parent.parent.label("bandwidth"), self.bandwidth)
        form_layout.addRow(self.parent.parent.label("spread_factor"), self.spread_factor)
        form_layout.addRow(self.parent.parent.label("coding_rate"), self.coding_rate)
        form_layout.addRow(self.parent.parent.label(""), self.psk_random_button)
        form_layout.addRow(self.tr(""), self.psk_default_button)
        if self.channel_index > 0:
            form_layout.addRow(self.tr(""), self.delete_this_channel_button)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def reset_form(self):
        """Reset form"""
        print('reset form')
        self.name.setText("")
        self.name.setFocus()
        self.role.setCurrentIndex(0)
        self.uplink_enabled.setChecked(False)
        self.downlink_enabled.setChecked(False)
        self.tx_power.setText("0")
        self.bandwidth.setText("0")
        self.spread_factor.setText("0")
        self.coding_rate.setText("0")


    def delete_this_channel(self):
        """Delete this channel"""
        print('delete this channel')
        self.interface.localNode.deleteChannel(self.channel_index)
        self.reset_form()
        self.get_values()


    def psk_random(self):
        """Generate random psk"""
        print('generated random psk')
        self.psk = fromPSK("random")
        print(f'psk is now:{self.psk}')


    def psk_default(self):
        """Use default psk"""
        print('using default psk')
        self.psk = fromPSK("default")
        print(f'psk is now:{self.psk}')


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface:

                # The python lib does not allow "gaps" in the index's
                # So, we "simulate" that here by only enabling the name only if the prior channel has a name
                if self.channel_index > 1:
                    prev_channel_form = self.parent.channel_forms[self.channel_index - 1]
                    if prev_channel_form.name.text() == '':
                        self.name.setReadOnly(True)
                    else:
                        self.name.setReadOnly(False)


                self.ch = self.interface.localNode.getChannelByChannelIndex(self.channel_index)
                print(f'self.ch:{self.ch}')

                if self.ch.settings.name:
                    self.name.setText(f'{self.ch.settings.name}')
                    if self.channel_index > 0:
                        self.delete_this_channel_button.show()
                else:
                    self.name.setText("")
                    if self.channel_index > 0:
                        self.delete_this_channel_button.hide()
                if self.channel_index == 0:
                    self.name.setText("PRIMARY")

                temp = 0
                if self.ch.role:
                    temp = self.ch.role
                self.role.clear()
                desc = Channel.Role.DESCRIPTOR
                for k,v in desc.values_by_name.items():
                    if self.channel_index == 0:
                        # only allow primary channel role on primary channel
                        if v.number == 1:
                            self.role.addItem(k, v.number)
                            self.role.setCurrentIndex(0)
                    else:
                        if v.number == 0:
                            self.role.addItem(k, v.number)
                            if v.number == temp:
                                self.role.setCurrentIndex(0)
                        elif v.number == 1:
                            pass
                        elif v.number == 2:
                            self.role.addItem(k, v.number)
                            if v.number == temp:
                                self.role.setCurrentIndex(1)

                if self.modem_config:
                    temp = 0
                    if self.ch.settings.modem_config:
                        temp = self.ch.settings.modem_config
                    self.modem_config.clear()
                    desc = ChannelSettings.ModemConfig.DESCRIPTOR
                    for k,v in desc.values_by_name.items():
                        self.modem_config.addItem(k, v.number)
                        if v.number == temp:
                            self.modem_config.setCurrentIndex(v.number)

                if self.ch.settings.psk:
                    self.psk = self.ch.settings.psk

                if self.ch.settings.uplink_enabled and self.ch.settings.uplink_enabled is True:
                    self.uplink_enabled.setChecked(True)

                if self.ch.settings.downlink_enabled and self.ch.settings.downlink_enabled is True:
                    self.downlink_enabled.setChecked(True)

                if self.ch.settings.tx_power:
                    self.tx_power.setText(f'{self.tx_power.text()}')
                else:
                    self.tx_power.setText("0")

                if self.ch.settings.bandwidth:
                    self.bandwidth.setText(f'{self.bandwidth.text()}')
                else:
                    self.bandwidth.setText("0")

                if self.ch.settings.spread_factor:
                    self.spread_factor.setText(f'{self.spread_factor.text()}')
                else:
                    self.spread_factor.setText("0")

                if self.ch.settings.coding_rate:
                    self.coding_rate.setText(f'{self.coding_rate.text()}')
                else:
                    self.coding_rate.setText("0")
        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:

                self.use_name = self.name.text()

                if self.use_name == '':
                    QMessageBox.warning(self, "Warning", "Need to have a channel name")
                else:
                    # Primary channel stuff
                    if self.channel_index == 0:
                        self.use_name = "" # primary has no name

                    before = self.interface.localNode.getChannelByChannelIndex(self.channel_index)
                    print(f'before:{before}')

                    ch = Channel()
                    ch.role = self.role.currentData()
                    ch.settings.name = self.use_name
                    if self.channel_index == 0:
                        # ex: chs.modem_config = chs.Bw250Cr46Sf2048
                        ch.settings.modem_config = self.modem_config.currentData()
                    if self.psk is None and self.channel_index != 0:
                        # if there was no PSK (i.e., adding a secondary channel then generate a PSK)
                        self.psk = genPSK256()
                    ch.settings.psk = self.psk
                    ch.settings.uplink_enabled = self.uplink_enabled.isChecked()
                    ch.settings.downlink_enabled = self.downlink_enabled.isChecked()
                    ch.settings.tx_power = int(zero_if_blank(self.tx_power.text()))
                    ch.settings.bandwidth = int(zero_if_blank(self.bandwidth.text()))
                    ch.settings.spread_factor = int(zero_if_blank(self.spread_factor.text()))
                    ch.settings.coding_rate = int(zero_if_blank(self.coding_rate.text()))
                    ch.index = self.channel_index
                    print(f'ch:{ch}')

                    n = self.interface.getNode(BROADCAST_ADDR)

                    print(f'before n.channels:{n.channels}')
                    n.channels[self.channel_index] = ch
                    print(f'after n.channels:{n.channels}')

                    print("Writing modified channels to device")
                    n.writeChannel(self.channel_index)

                    # re-get the values (so the "Delete this channel" button will show/hide when appropriate)
                    self.get_values()

        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()



"""class for the power settings"""


from PySide6.QtWidgets import QDialog, QCheckBox, QFormLayout, QComboBox, QDialogButtonBox, QLineEdit

import meshtastic.serial_interface
import meshtastic.util
import meshtastic.mesh_pb2
import meshtastic.radioconfig_pb2
from meshtastic.__init__ import BROADCAST_ADDR
from meshtastic.__main__ import setPref
from meshtastic_flasher.util import zero_if_blank


class PowerForm(QDialog):
    """position settings form"""

    def __init__(self, parent=None):
        """constructor"""
        super(PowerForm, self).__init__(parent)

        self.parent = parent

        width = 500
        height = 200
        self.setMinimumSize(width, height)
        self.setWindowTitle("Power Settings")

        self.port = None
        self.interface = None
        self.prefs = None

        # Create widgets
        self.charge_current = QComboBox()
        self.charge_current.setToolTip(self.parent.description('charge_current'))
        self.charge_current.setMinimumContentsLength(17)
        self.is_always_powered = QCheckBox()
        self.is_always_powered.setToolTip(self.parent.description('is_always_powered'))
        self.is_low_power = QCheckBox()
        self.is_low_power.setToolTip(self.parent.description('is_low_power'))
        self.adc_multiplier_override = QLineEdit()
        self.adc_multiplier_override.setToolTip(self.parent.description('adc_multiplier_override'))
        self.is_power_saving = QCheckBox()
        self.is_power_saving.setToolTip(self.parent.description('is_power_saving'))
        self.ls_secs = QLineEdit()
        self.ls_secs.setToolTip(self.parent.description('ls_secs'))
        self.mesh_sds_timeout_secs = QLineEdit()
        self.mesh_sds_timeout_secs.setToolTip(self.parent.description('mesh_sds_timeout_secs'))
        self.min_wake_secs = QLineEdit()
        self.min_wake_secs.setToolTip(self.parent.description('min_wake_secs'))
        self.on_battery_shutdown_after_secs = QLineEdit()
        self.on_battery_shutdown_after_secs.setToolTip(self.parent.description('on_battery_shutdown_after_secs'))
        self.phone_sds_timeout_sec = QLineEdit()
        self.phone_sds_timeout_sec.setToolTip(self.parent.description('phone_sds_timeout_sec'))
        self.phone_timeout_secs = QLineEdit()
        self.phone_timeout_secs.setToolTip(self.parent.description('phone_timeout_secs'))
        self.screen_on_secs = QLineEdit()
        self.screen_on_secs.setToolTip(self.parent.description('screen_on_secs'))
        self.sds_secs = QLineEdit()
        self.sds_secs.setToolTip(self.parent.description('sds_secs'))
        self.wait_bluetooth_secs = QLineEdit()
        self.wait_bluetooth_secs.setToolTip(self.parent.description('wait_bluetooth_secs'))

        # Add a button box
        self.button_box = QDialogButtonBox()
        self.button_box.setStandardButtons(QDialogButtonBox.Save)
        self.button_box.accepted.connect(self.accept)
        self.button_box.rejected.connect(self.reject)

        # create form
        form_layout = QFormLayout()
        form_layout.addRow(self.parent.label("charge_current"), self.charge_current)
        form_layout.addRow(self.parent.label("is_always_powered"), self.is_always_powered)
        form_layout.addRow(self.parent.label("is_low_power"), self.is_low_power)
        form_layout.addRow(self.parent.label("adc_multiplier_override"), self.adc_multiplier_override)
        form_layout.addRow(self.parent.label("is_power_saving"), self.is_power_saving)
        form_layout.addRow(self.parent.label("ls_secs"), self.ls_secs)
        form_layout.addRow(self.parent.label("mesh_sds_timeout_secs"), self.mesh_sds_timeout_secs)
        form_layout.addRow(self.parent.label("min_wake_secs"), self.min_wake_secs)
        form_layout.addRow(self.parent.label("on_battery_shutdown_after_secs"), self.on_battery_shutdown_after_secs)
        form_layout.addRow(self.parent.label("phone_sds_timeout_sec"), self.phone_sds_timeout_sec)
        form_layout.addRow(self.parent.label("phone_timeout_secs"), self.phone_timeout_secs)
        form_layout.addRow(self.parent.label("screen_on_secs"), self.screen_on_secs)
        form_layout.addRow(self.parent.label("sds_secs"), self.sds_secs)
        form_layout.addRow(self.parent.label("wait_bluetooth_secs"), self.wait_bluetooth_secs)
        form_layout.addRow(self.tr(""), self.button_box)
        self.setLayout(form_layout)


    def run(self, port=None, interface=None):
        """load the form"""
        self.port = port
        self.interface = interface
        if self.port:
            print(f'using port:{self.port}')
            self.get_values()
            self.show()


    def get_values(self):
        """Get values from device"""
        try:
            if self.interface is None:
                print('interface was none?')
                self.interface = meshtastic.serial_interface.SerialInterface(devPath=self.port)
            if self.interface:
                self.prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences

                temp = 0
                if self.prefs.charge_current:
                    temp = self.prefs.charge_current
                self.charge_current.clear()
                desc = meshtastic.radioconfig_pb2.ChargeCurrent.DESCRIPTOR
                for k,v in desc.values_by_name.items():
                    self.charge_current.addItem(k, v.number)
                    if v.number == temp:
                        self.charge_current.setCurrentIndex(v.number)

                if self.prefs.is_always_powered and self.prefs.is_always_powered is True:
                    self.is_always_powered.setChecked(True)

                if self.prefs.is_low_power and self.prefs.is_low_power is True:
                    self.is_low_power.setChecked(True)

                if self.prefs.adc_multiplier_override:
                    self.adc_multiplier_override.setText(f'{self.prefs.adc_multiplier_override}')
                else:
                    self.adc_multiplier_override.setText("0")

                if self.prefs.is_power_saving and self.prefs.is_power_saving is True:
                    self.is_power_saving.setChecked(True)

                if self.prefs.ls_secs:
                    self.ls_secs.setText(f'{self.prefs.ls_secs}')
                else:
                    self.ls_secs.setText("0")

                if self.prefs.mesh_sds_timeout_secs:
                    self.mesh_sds_timeout_secs.setText(f'{self.prefs.mesh_sds_timeout_secs}')
                else:
                    self.mesh_sds_timeout_secs.setText("0")

                if self.prefs.min_wake_secs:
                    self.min_wake_secs.setText(f'{self.prefs.min_wake_secs}')
                else:
                    self.min_wake_secs.setText("0")

                if self.prefs.on_battery_shutdown_after_secs:
                    self.on_battery_shutdown_after_secs.setText(f'{self.prefs.on_battery_shutdown_after_secs}')
                else:
                    self.on_battery_shutdown_after_secs.setText("0")

                if self.prefs.phone_sds_timeout_sec:
                    self.phone_sds_timeout_sec.setText(f'{self.prefs.phone_sds_timeout_sec}')
                else:
                    self.phone_sds_timeout_sec.setText("0")

                if self.prefs.phone_timeout_secs:
                    self.phone_timeout_secs.setText(f'{self.prefs.phone_timeout_secs}')
                else:
                    self.phone_timeout_secs.setText("0")

                if self.prefs.screen_on_secs:
                    self.screen_on_secs.setText(f'{self.prefs.screen_on_secs}')
                else:
                    self.screen_on_secs.setText("0")

                if self.prefs.sds_secs:
                    self.sds_secs.setText(f'{self.prefs.sds_secs}')
                else:
                    self.sds_secs.setText("0")

                if self.prefs.wait_bluetooth_secs:
                    self.wait_bluetooth_secs.setText(f'{self.prefs.wait_bluetooth_secs}')
                else:
                    self.wait_bluetooth_secs.setText("0")

        except Exception as e:
            print(f'Exception:{e}')


    def write_values(self):
        """Write values to device"""
        try:
            if self.interface:
                print("Writing preferences to device")
                prefs = self.interface.getNode(BROADCAST_ADDR).radioConfig.preferences
                setPref(prefs, 'charge_current', f'{self.charge_current.currentData()}')
                setPref(prefs, 'is_always_powered', f'{self.is_always_powered.isChecked()}')
                setPref(prefs, 'is_low_power', f'{self.is_low_power.isChecked()}')
                setPref(prefs, 'adc_multiplier_override', zero_if_blank(self.adc_multiplier_override.text()))
                setPref(prefs, 'is_power_saving', f'{self.is_power_saving.isChecked()}')
                setPref(prefs, 'ls_secs', zero_if_blank(self.ls_secs.text()))
                setPref(prefs, 'mesh_sds_timeout_secs', zero_if_blank(self.mesh_sds_timeout_secs.text()))
                setPref(prefs, 'min_wake_secs', zero_if_blank(self.min_wake_secs.text()))
                setPref(prefs, 'on_battery_shutdown_after_secs', zero_if_blank(self.on_battery_shutdown_after_secs.text()))
                setPref(prefs, 'phone_sds_timeout_sec', zero_if_blank(self.phone_sds_timeout_sec.text()))
                setPref(prefs, 'phone_timeout_secs', zero_if_blank(self.phone_timeout_secs.text()))
                setPref(prefs, 'screen_on_secs', zero_if_blank(self.screen_on_secs.text()))
                setPref(prefs, 'sds_secs', zero_if_blank(self.sds_secs.text()))
                setPref(prefs, 'wait_bluetooth_secs', zero_if_blank(self.wait_bluetooth_secs.text()))
                self.interface.getNode(BROADCAST_ADDR).writeConfig()

        except Exception as e:
            print(f'Exception:{e}')


    def reject(self):
        """Cancel without saving"""
        print('CANCEL button was clicked')
        self.parent.my_close()


    def accept(self):
        """Close the form"""
        print('SAVE button was clicked')
        self.write_values()
